% -------------------------------------------------------------------
% ckbs: Constrained Kalman-Bucy Smoother Program: Copyright (C) 2006
% Authors: Bradlely Bell:        bradbell at washington dot edu
%          Gianluigi Pillonetto: giapi at dei dot unipd dot it
%          Aleksandr Y. Aravkin: sasha.aravkin at gmail dot com
%          James V. Burke:       burke at math dot washington dot edu
% License: GNU General Public License Version 2
% -------------------------------------------------------------------
function [ok] = sumsq_grad_ok()
N    = 3;
m    = 2;
n    = 2;
x    = zeros(n, N);
z    = [ ...
   0.050323  -0.411249  -0.998710
  -4.750912  -5.552283  -5.814256
];
g    = [ ...
   0.1874033   0.0025807  -0.0075234
   4.5928281  -0.0098694  -0.0128231
];
h    = [ ...
  -0.18740  -0.64927  -1.18680
  -4.59283  -5.45052  -6.00335
];
dg   = zeros(n, n, N);
dg(:,:,2)    = [ ...
   1.00000   0.10000
  -1.36285   1.19298
];
dg(:,:,3)    = [ ...
   1.00000   0.10000
  -2.60564   1.11569
];
dh        = zeros(m, n, N);
dh(:,:,1) = eye(2);
dh(:,:,2) = eye(2);
dh(:,:,3) = eye(2);
qinv = zeros(n, n, N);
qinv(:,:,1)    = [ ...
   0.010000   0.000000
   0.000000   0.010000
];
qinv(:,:,2)    = [ ...
   100.00000     0.00000
     0.00000   100.00000
];
qinv(:,:,3)    = [ ...
   100.00000     0.00000
     0.00000   100.00000
];
rinv           = zeros(m, n, N);
rinv(:,:,1)    = 4. * eye(2);
rinv(:,:,2)    = 4. * eye(2);
rinv(:,:,3)    = 4. * eye(2);
%
addpath('../src')
grad  = ckbs_sumsq_grad(x, z, g, h, dg, dh, qinv, rinv);
%
check = zeros(n, N);
step = 1e-4;
for k = 1 : N
	for j = 1 : n
		xk_j        = x(j, k);
		x(j, k)     = xk_j - step;
		obj_m       = ckbs_sumsq_obj(x, z, g, h, dg, dh, qinv, rinv);
		x(j, k)     = xk_j + step;
		obj_p       = ckbs_sumsq_obj(x, z, g, h, dg, dh, qinv, rinv);
		check(j, k) = (obj_p - obj_m) / (2 * step);
		x(j, k)     = xk_j;
	end
end
%
ok = all( all( abs(grad - check) < 1e-3 ) );
